// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright (c) 2024 Patrick_Pluto

use clap::Parser;
use cpu::CPU;
use file::{load_binary_u8, load_file, save_binary_u8};
mod assembler;
mod cpu;
mod file;

/// SysForge: Systems programming made easy.
#[derive(Parser, Debug)]
#[command(version, about, long_about = None)]
struct Args {
    /// Assemble a file
    #[arg(short, long, default_value_t = false)]
    assemble: bool,

    /// Run the emulator
    #[arg(short, long, default_value_t = false)]
    emulate: bool,

    /// Selects the file
    #[arg(short, long, default_value = "main.asm")]
    file: String,

    /// Selects the output file
    #[arg(short, long, default_value = "main.out")]
    output: String,

    /// Selects which CPU to use
    #[arg(long, default_value = "ExampleCPU")]
    cpu: String,
}

fn main() {
    let args: Args = Args::parse();
    if args.assemble {
        save_binary_u8(
            &args.output,
            cpu::ExampleCPU::assemble(load_file(&args.file)),
        );
    } else if args.emulate {
        cpu::ExampleCPU::create(true, load_binary_u8(&args.file)).start();
    }
}
